package dicom;


import com.sun.media.imageioimpl.plugins.jpeg2000.ImageInputStreamWrapper;
import com.sun.media.imageio.stream.SegmentedImageInputStream;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.imageio.ImageIO;
import javax.imageio.stream.ImageInputStream;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.data.UID;
import org.dcm4che2.imageio.ItemParser;
import org.dcm4che2.io.DicomInputStream;
import org.dcm4che2.io.StopTagInputHandler;
import org.dcm4che2.data.Tag;

/**
 * Convertisor dcm to mpeg2
 * @author dimitri.pinaeta@gmail.com with help forum dcm4che2 (by boken)
 */
public class dcmTompg {


    public dcmTompg(File file) throws IOException{
     final ImageInputStream iis = ImageIO.createImageInputStream(file);//Reading file JPEG2000 or standard JPEG
        
try{
    final DicomInputStream dis = new DicomInputStream(iis);//Reading file dicom
    dis.setHandler(new StopTagInputHandler(Tag.PixelData));//No reading tag Pixel Data
    final DicomObject dicomObject = dis.readDicomObject();// Read metadata

    ItemParser itemParser = new ItemParser(dis, iis,1, UID.MPEG2);//ItemParser(DicomInputStream dis, javax.imageio.stream.ImageInputStream iis, int numberOfFrames, java.lang.String tsuid)
    final SegmentedImageInputStream siis = new SegmentedImageInputStream(iis,itemParser);//A SegmentedImageInputStream provides a view of a subset of another ImageInputStream consiting of a series of segments with given starting positions in the source stream and lengths. The resulting stream behaves like an ordinary ImageInputStream.

    itemParser.seekFrame(siis,0);//seekFrame(com.sun.media.imageio.stream.SegmentedImageInputStream siis, int frame) Choosing image
    final File newFile = new File(file.getAbsoluteFile().getParent(), dicomObject.getString(Tag.SOPInstanceUID)+".mpg");
    newFile.createNewFile();//Create new File
    FileOutputStream fos = new FileOutputStream(newFile);
try{// Block of code with multiple exit points

   transfer(new ImageInputStreamWrapper(siis),fos);//ImageInputStreamWrapper : exposes and ImageInputStream through an InputStream api
    
}

finally{// Block of code that is always executed when the try block is exited,
      // no matter how the try block is exited
fos.close();
}
}
        
 finally
 {
iis.close();
 }
 }
/**
 * This function is a utility to read all data from in and write out.
 * @param iis : input file
 * @param out : output file
 */
    public void transfer(InputStream in, OutputStream out) throws IOException{
        
byte[] buffer = new byte[65536];//65536
int read;
while((read = in.read(buffer))!=-1) //If the file (byte) is empty
    out.write(buffer,0,read); //Build new file
    }

}
